namespace BodyTracking
{
	public class PersonDisplayer
	{
		private KinectSensor sensor;

		private ColorFrameReader colorReader;
		private BodyFrameReader bodyReader;

		private Body[] bodies;

		public PersonDisplayer()
		{
			InitializeComponent();

			sensor = KinectSensor.GetDefault();
			sensor.Open();
            
			colorReader = sensor.ColorFrameSource.OpenReader();
			colorReader.FrameArrived += Reader_ColorFrameArrived;
            
			bodyReader = sensor.BodyFrameSource.OpenReader();
			bodyReader.FrameArrived += Reader_BodyFrameArrived;

			bodies = new Body[6];
		}

		void Reader_ColorFrameArrived(object sender, 
													ColorFrameArrivedEventArgs args)
		{
			var frame = args.FrameReference.AcquireFrame();

			if (frame != null)
			{
				camera.Source = ToBitmap(frame);
			}
		}

		private ImageSource ToBitmap(ColorFrame frame)
		{
			int width = frame.FrameDescription.Width;
			int height = frame.FrameDescription.Height;

			byte[] pixels = new byte[/*size of picture*/];

			if (frame.RawColorImageFormat==ColorImageFormat.Bgra)
			{
				frame.CopyRawFrameDataToArray(pixels);
			}
			else
			{
				frame.CopyConvertedFrameDataToArray(pixels, 
																		ColorImageFormat.Bgra);
			}

			/*create bitmapsource and return it*/
		}     

		void Reader_BodyFrameArrived(object sender, 
												 BodyFrameArrivedEventArgs args)
		{
			var frame = args.FrameReference.AcquireFrame();

			if (frame != null)
			{
				frame.GetAndRefreshBodyData(bodies);

				if (headsCanvas != null)
				{
					headsCanvas.Children.Clear();
				}

				foreach (Body body in bodies)
				{
					if (body.IsTracked)
					{
						DrawBody(body);
					}
				}
			}
		}

		private void DrawBody(Body body)
		{
			foreach (JointType joint in body.Joints.Keys)
			{
				DrawJoint(body.Joints[joint]);
			}
		}

		private void DrawJoint(Joint joint)
		{
			/*get joint position*/
 
			Ellipse ellipse = new Ellipse();
			ellipse.Fill = Brushes.RoyalBlue;
			ellipse.Stroke = Brushes.White;
			ellipse.StrokeThickness = borderWidth;
			ellipse.Width = 15;
			ellipse.Height = 2;

			/*position ellipse at joint position*/

			headsCanvas.Children.Add(ellipse);
		}
	}
}


