﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Media.Imaging;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Media;

namespace BodyTracking
{
    class FrameReceiver : IPositionInfoReleased, ICameraUpdate, IConnectionFailed
    {
        int filen = 0;

        //colorframe size
        private const int ColorFrameWidth = 1920;
        private const int ColorFrameHeight = 1080;

        //eventhandler tracking data
        event EventHandler newPositonInfoEvent;
        object newPositonInfoEventLock = new Object();

        event EventHandler IPositionInfoReleased.OnNewPositionInfoArrived
        {
            add
            {
                lock (newPositonInfoEventLock)
                {
                    newPositonInfoEvent += value;
                }
            }
            remove
            {
                lock (newPositonInfoEventLock)
                {
                    newPositonInfoEvent -= value;
                }
            }
        }

        //eventhandler camera update
        event EventHandler cameraEvent;
        object cameraEventLock = new Object();

        event EventHandler ICameraUpdate.OnFrameArrived
        {
            add
            {
                lock (cameraEventLock)
                {
                    cameraEvent += value;
                }
            }
            remove
            {
                lock (cameraEventLock)
                {
                    cameraEvent -= value;
                }
            }
        }

        //eventhandler connection failed
        event EventHandler connectionFailedEvent;
        object connectionFailedEventLock = new Object();

        event EventHandler IConnectionFailed.OnConnectionFailed
        {
            add
            {
                lock (connectionFailedEventLock)
                {
                    connectionFailedEvent += value;
                }
            }
            remove
            {
                lock (connectionFailedEventLock)
                {
                    connectionFailedEvent -= value;
                }
            }
        }

        enum FrameType
        {
            body = 0,
            color = 1,
            depth = 2,
            infrared = 3
        }

        //network
        private TcpClient tcpClient;
        private IPEndPoint serverEndPoint = new IPEndPoint(IPAddress.Parse("10.0.0.6"), 8001);

        //threads
        private Thread frameReceiver;

        //messages
        byte[] bodyframemessage = { Convert.ToByte('b'), Convert.ToByte('o'), Convert.ToByte('d'), Convert.ToByte('y') };
        byte[] colorframemessage = { Convert.ToByte('c'), Convert.ToByte('o'), Convert.ToByte('l'), Convert.ToByte('o'), Convert.ToByte('r') };

        private bool setup;

        public FrameReceiver()
        {
            try
            {
                tcpClient = new TcpClient();

                Console.WriteLine("Connecting to TCP server at IP: " + serverEndPoint.Address.ToString());
                if (AttemptConnectToServer() == -1)
                {
                    setup = false;

                    EventHandler connectionEventHandler = connectionFailedEvent;
                    if (connectionEventHandler != null)
                    {
                        byte[] m = { Convert.ToByte('f'), Convert.ToByte('r'), Convert.ToByte('a'), Convert.ToByte('m'), Convert.ToByte('e') };
                        connectionEventHandler(this, new ConnectionFailedEventArgs(m));
                    }
                    return;
                }
                Console.WriteLine("Connect to TCP server succeeded");

                frameReceiver = new Thread(new ThreadStart(ReceiveFrames));
                frameReceiver.Start();

                setup = true;
            }
            catch
            {
                return;
            }

        }

        private int AttemptConnectToServer()
        {
            TimeSpan starttime = (TimeSpan)DateTime.Now.TimeOfDay;
            TimeSpan interval = new TimeSpan(0, 0, 6);

            bool connected = false;
            bool timedout = false;

            while (connected == false && timedout == false)
            {
                if ((TimeSpan)DateTime.Now.TimeOfDay < (starttime + interval))
                {
                    connected = ConnectToServer();
                }
                else
                {
                    timedout = true;
                }
            }

            if (connected == false)
            {
                return -1;
            }
            else
            {
                return 0;
            }
        }

        private bool ConnectToServer()
        {
            try
            {
                if (tcpClient.Connected)
                {
                    tcpClient.Close();
                }

                tcpClient = null;
                tcpClient = new TcpClient();

                tcpClient.Connect(serverEndPoint);

                NetworkStream stream = tcpClient.GetStream();

                stream.Write(bodyframemessage, 0, bodyframemessage.Length);
                stream.Flush();

                stream.Write(colorframemessage, 0, colorframemessage.Length);
                stream.Flush();
            }
            catch
            {
                return false;
            }
            return true;
        }

        private void ReceiveFrames()
        {
            int numberOfPeople = 6;
            int slotsPerPerson = 4;

            NetworkStream stream = tcpClient.GetStream();

            int messagelength = ColorFrameWidth * ColorFrameHeight * ((PixelFormats.Bgr32.BitsPerPixel + 7) / 8);
            byte[] bytemessage;
            int bytesRead;

            while (true)
            {
                bytemessage = new byte[messagelength + 1];
                bytesRead = 0;

                try
                {
                    bytesRead = stream.Read(bytemessage, 0, messagelength);
                }
                catch
                {
                    Console.WriteLine("catch");
                }

                if (CheckIfBodyData(bytemessage) == true)
                {
                    byte[] message = new byte[numberOfPeople * slotsPerPerson];
                    message = GetPartOfMessage(bytemessage, 1, numberOfPeople * slotsPerPerson);

                    EventHandler newPositionInfoEventHandler = newPositonInfoEvent;
                    if (newPositionInfoEventHandler != null)
                    {
                        newPositionInfoEventHandler(this, new PositionInfoEventArgs(message));
                    }
                }
                else
                {
                    byte[] message = new byte[messagelength];
                    message = GetPartOfMessage(bytemessage, 1, messagelength);

                    EventHandler eventHandler = cameraEvent;
                    if (eventHandler != null)
                    {
                        eventHandler(this, new CameraEventArgs(message));
                    }
                }

            }
        }

        private bool CheckIfBodyData(byte[] m)
        {
            if (m[0] != (byte)FrameType.body)
            {
                return true;
            }

            return false;
        }

        byte[] GetPartOfMessage(byte[] m, int start, int end)
        {
            byte[] message = new byte[end - start];
            for (int i = 0; i < message.Length; i++)
            {
                if ((i + start) < m.Length)
                {
                    message[i] = m[i + start];
                }
            }
            return message;
        }

        void PrintMessage(byte[] message)
        {
            for (int i = 0; i < message.Length; i++)
            {
                Console.Write(message[i]);
            }

            Console.Write("\n");

        }

    }
}
