﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;

namespace BodyTracking
{
    partial class AppWindow : Window
    {
        //colorframe size
        private const int ColorFrameWidth = 1920;
        private const int ColorFrameHeight = 1080;

        private PersonDisplayer personDisplayer;

        private ICameraUpdate cameraUpdater;
        private ICanvasUpdate canvasUpdater;

        private BitmapImage bitmap;

        private Thread LoadCameraImageThread;

        public bool new_image;

        public byte[] colorimage;

        public AppWindow()
        {
            InitializeComponent();

            personDisplayer = new PersonDisplayer();
      
            cameraUpdater = (ICameraUpdate)personDisplayer;
            cameraUpdater.OnFrameArrived += new EventHandler(UpdateCameraEvent);
            canvasUpdater = (ICanvasUpdate)personDisplayer;
            canvasUpdater.OnCanvasDataArrived += new EventHandler(UpdateCanvasEvent);

            bitmap = null;

            LoadCameraImageThread = new Thread(new ThreadStart(LoadCameraImage));
            LoadCameraImageThread.Start();

            bitmap = new BitmapImage();
            camera.Source = bitmap;

            new_image = false;

            colorimage = new byte[ColorFrameWidth * ColorFrameHeight * ((PixelFormats.Bgra32.BitsPerPixel + 7) / 8)];
        }

        void UpdateCameraEvent(object sender, EventArgs args)
        {
            CameraEventArgs cameraargs = (CameraEventArgs)args;

            colorimage = cameraargs.message;

            new_image = true;
        }

        void UpdateCanvasEvent(object o, EventArgs args)
        {
            CanvasDataEventArgs cdargs = (CanvasDataEventArgs)args;
            this.Dispatcher.Invoke(new Action(() =>
            {
                headCanvas.Children.Clear();

                for (int i = 0; i < 6; i++)
                {
                    if (cdargs.message[i * 4] != 0)
                    {
                        DrawHeadSquare(cdargs.message[i * 4 + 1], cdargs.message[i * 4 + 2], Brushes.Green);
                    }

                }
            }));
        }

        void LoadCameraImage()
        {
            while(false)
            {
                if (new_image == true)
                {
                    try { 

                        this.Dispatcher.Invoke(new Action(() =>
                        {
                            //MemoryStream stream = new MemoryStream();
                            int stride = ColorFrameWidth * ((PixelFormats.Bgr32.BitsPerPixel + 7) / 8);

                            if (colorimage != null)// & colorimage.Length > 0)
                            {
                                /*stream.Write(colorimage, 0, colorimage.Length);

                                bitmap = new BitmapImage();

                                bitmap.BeginInit();
                                bitmap.CacheOption = BitmapCacheOption.OnLoad;
                                bitmap.StreamSource = stream;
                                bitmap.EndInit();
                                stream.Close();
                                stream.Dispose();*/

                                //bitmap = (BitmapImage)BitmapSource.Create(ColorFrameWidth, ColorFrameHeight, 96, 96, PixelFormats.Bgr32, null, colorimage, stride);
                            }

                            BitmapSource bitmapsource = BitmapSource.Create(ColorFrameWidth, ColorFrameHeight, 96, 96, PixelFormats.Bgr32, null, colorimage, stride);
                            camera.Source = bitmapsource; // BitmapSource.Create(ColorFrameWidth, ColorFrameHeight, 96, 96, PixelFormats.Bgr32, null, colorimage, stride);
                            
                            bitmapsource = null;
                            colorimage = null;
                            //new_image = false;

                        }));
                    }
                    catch
                    {
                        //something went wrong
                    }
                }
            }
        }

        private Body ClickInsideSquare(int x, int y)
        {
            int bodynr = 0;

            try
            {
                foreach (Rectangle rectangle in headCanvas.Children)
                {
                    int rectangleHeight = (int)rectangle.Height;
                    int rectangleWidth = (int)rectangle.Width;
                    int rectangleX = (int)rectangle.GetValue(Canvas.LeftProperty);
                    int rectangleY = (int)rectangle.GetValue(Canvas.TopProperty);

                    if ((x > rectangleX) && (x < (rectangleX + rectangleWidth)) &&
                        (y > rectangleY) && (y < (rectangleY + rectangleHeight)))
                    {
                        //return bodies[bodynr];
                    }
                    bodynr++;
                }
                return null;
            }
            catch
            {
                return null;
            }
        }

        private void DrawHeadSquare(int posx, int posy, SolidColorBrush border)
        {
            int size = 10;
            Rectangle rectangle = new Rectangle();
            rectangle.StrokeThickness = 5;
            rectangle.Stroke = border;
            rectangle.Height = size;
            rectangle.Width = size;

            var screenWidth = 1050;
            var screenHeight = 700;

            Canvas.SetLeft(rectangle, (posx / 1920) * screenWidth - size / 2);
            Canvas.SetTop(rectangle, (posy / 1080) * screenHeight - size / 2);

            headCanvas.Children.Add(rectangle);
        }

        private void Mouse_ClickEvent(object sender, MouseButtonEventArgs args)
        {
            /*Body clicked_body = null;
            var point = args.GetPosition(camera);
            int x = (int)point.X;
            int y = (int)point.Y;
            if ((clicked_body = ClickInsideSquare(x, y)) != null)
            {
                tracked_body = clicked_body;
            }*/
            //displayer.updateTrackedBody(int x, int y);
        }
    }

}