﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;

namespace BodyTracking
{

    public struct Person
    {
        public ulong ID;
        public int x;
        public int y;
        public int z;

        public Person(ulong id, int posx, int posy, int dist)
        {
            ID = id;
            x = posx;
            y = posy;
            z = dist;
        }
    }

    public class PersonDisplayer : ICameraUpdate, ICanvasUpdate
    {
        //colorframe size
        private const int RGBpixelWidth = 1920;
        private const int RGBpixelHeight = 1080;

        //eventhandler camera update
        event EventHandler cameraEvent;
        object cameraEventLock = new Object();

        event EventHandler ICameraUpdate.OnFrameArrived
        {
            add
            {
                lock (cameraEventLock)
                {
                    cameraEvent += value;
                }
            }
            remove
            {
                lock (cameraEventLock)
                {
                    cameraEvent -= value;
                }
            }
        }

        //eventhandler canvas update
        event EventHandler canvasEvent;
        object canvasEventLock = new Object();

        event EventHandler ICanvasUpdate.OnCanvasDataArrived
        {
            add
            {
                lock (canvasEventLock)
                {
                    canvasEvent += value;
                }
            }
            remove
            {
                lock (canvasEventLock)
                {
                    canvasEvent -= value;
                }
            }
        }

        private Person[] people;
        private Person trackedPerson;
        private ulong trackedPersonid;

        FrameReceiver frameReceiver;
        TrackingdataServer trackingdataServer;

        private IPositionInfoReleased positionDataUpdater;
        private ICameraUpdate cameraUpdater;

        public PersonDisplayer()
        {
            people = new Person[6];
            trackedPerson = new Person(0,0,0,0);

            frameReceiver = new FrameReceiver();
            trackingdataServer = new TrackingdataServer();

            positionDataUpdater = (IPositionInfoReleased)frameReceiver;
            positionDataUpdater.OnNewPositionInfoArrived += new EventHandler(UpdatePositionDataEvent);

            cameraUpdater = (ICameraUpdate)frameReceiver;
            cameraUpdater.OnFrameArrived += new EventHandler(UpdateCameraEvent);

        }

        public void SetTrackedBody(Person newBody, ulong newID)
        {
            trackedPerson = newBody;
            trackedPersonid = newID;
        }

        void UpdatePositionDataEvent(object o, EventArgs args)
        {
            PositionInfoEventArgs piargs = (PositionInfoEventArgs)args;

            UpdatePositionInfo(piargs.message);

            EventHandler eventHandler = canvasEvent;
            if (eventHandler != null)
            {
                eventHandler(this, new CanvasDataEventArgs(piargs.message));
            }
        }

        void UpdateCameraEvent(object o, EventArgs args)
        {
            CameraEventArgs cargs = (CameraEventArgs)args;

            EventHandler eventHandler = cameraEvent;
            if (eventHandler != null)
            {
                eventHandler(this, new CameraEventArgs(cargs.message));
            }
        }

        private void UpdatePositionInfo(byte[] input)
        {
            int slotsPerPerson = 4;
            if (input != null)
            {
                people = new Person[input.Length / slotsPerPerson];
                for (int i = 0; i < input.Length / slotsPerPerson; i++)
                {
                    people[i] = new Person(input[i * slotsPerPerson],
                                           input[i * slotsPerPerson + 1],
                                           input[i * slotsPerPerson + 2],
                                           input[i * slotsPerPerson + 3]);

                    if (trackedPerson.ID == input[i * slotsPerPerson])
                    {
                        trackedPerson.x = input[i * slotsPerPerson + 1];
                        trackedPerson.y = input[i * slotsPerPerson + 2];
                        trackedPerson.z = input[i * slotsPerPerson + 3];
                    }
                }
            }
        }

    }
}