﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Microsoft.Kinect;

namespace DecisionMaker
{

    struct Person
    {
        public ulong ID;
        public int x;
        public int y;
        public int z;

        public Person(ulong id, ulong posx, ulong posy, ulong dist){
            ID = id;
            x = (int)posx;
            y = (int)posy;
            z = (int)dist;
        }
    }

    class DecisionMaker
    {
        private TrackingdataServer trackingdataServer;
        private FrameReceiver frameReceiver;

        private IPositionInfoReleased positionDataUpdater;
        private IConnectionFailed trackingdataSupplierConnectionFailed;
        private IConnectionFailed frameReceiverConnectionFailed;

        private Random rand;

        private Person[] people;

        private Person trackedPerson;
        private ulong trackedPersonID;

        public byte[] data;
        public bool new_data;

        private Thread trackingdataSender;

        public DecisionMaker()
        {
            Console.WriteLine("-------------------------------------------------\n" +
                              "     Setup of module DECISION MAKER initiated    \n" +
                              "-------------------------------------------------\n");

            people = null;
            rand = new Random();

            Console.WriteLine("--------------------------------------------\n" +
                              "  Setup of tracking data supplier initiated \n" +
                              "---------------------------------------------\n");

            trackingdataServer = new TrackingdataServer();

            if (trackingdataServer.CheckSetup() == false)
            {
                PrintClosingMessage();
                return;
            }

            Console.WriteLine("--------------------------------------------\n" +
                              " Setup of tracking data supplier succeeded \n" +
                              "---------------------------------------------\n");
            
            Console.WriteLine("--------------------------------------------\n" +
                              "    Setup of frame receiver initiated   \n" +
                              "---------------------------------------------\n");

            frameReceiver = new FrameReceiver();

            if (frameReceiver.CheckSetup() == false)
            {
                PrintClosingMessage();
                return;
            }

            frameReceiverConnectionFailed = (IConnectionFailed)frameReceiver;
            frameReceiverConnectionFailed.OnConnectionFailed += new EventHandler(ConnectionFailed);

            positionDataUpdater = (IPositionInfoReleased)frameReceiver;
            positionDataUpdater.OnNewPositionInfoArrived += new EventHandler(NewPositionDataArrived);
            new_data = false;

            trackingdataSupplierConnectionFailed = (IConnectionFailed)trackingdataServer;
            trackingdataSupplierConnectionFailed.OnConnectionFailed += new EventHandler(ConnectionFailed);

            Console.WriteLine("--------------------------------------------\n" +
                              "    Setup of frame receiver succeeded   \n" +
                              "---------------------------------------------\n");

            Console.WriteLine("Starting up tracking data sending");

            //trackingdataSender = new Thread(new ThreadStart(SendTrackingdata));
            //trackingdataSender.Start();

            Console.WriteLine("Sending of tracking data started");

            new_data = false;

            Console.WriteLine("-------------------------------------------------\n" +
                              "     Setup of module DECISION MAKER completed    \n" +
                              "-------------------------------------------------");
        }

        private void PrintClosingMessage()
        {
            Console.Write("Could not connect. Exiting");
            Thread.Sleep(500);
            Console.Write(".");
            Thread.Sleep(500);
            Console.Write(".");
            Thread.Sleep(500);
            Console.Write(".");
            Thread.Sleep(1500);
        }
        
        void NewPositionDataArrived(object o, EventArgs args)
        {
            PositionInfoEventArgs nargs = (PositionInfoEventArgs)args;

            UpdateInfo(nargs.message);
        }

        void ConnectionFailed(object o, EventArgs args)
        {
            ConnectionFailedEventArgs cfargs = (ConnectionFailedEventArgs)args;

            for (int i = 0; i < cfargs.message.Length; i++)
            {
                Console.Write(cfargs.message[i]);
            }
            Console.Write("\n");

            PrintClosingMessage();
            Environment.Exit(0);
            return;
        }

        public void UpdateInfo(byte[] input)
        {
            int slotsPerPerson = 4;
            if (input != null)
            {
                people = new Person[input.Length / slotsPerPerson];
                for (int i = 0; i < input.Length / slotsPerPerson; i++)
                {
                    people[i] = new Person(input[i * slotsPerPerson], 
                                           input[i * slotsPerPerson + 1], 
                                           input[i * slotsPerPerson + 2],
                                           input[i * slotsPerPerson + 3]);

                    if (trackedPerson.ID == input[i * slotsPerPerson])
                    {
                        trackedPerson.x = input[i * slotsPerPerson + 1];
                        trackedPerson.y = input[i * slotsPerPerson + 2];
                        trackedPerson.z = input[i * slotsPerPerson + 3];
                    }
                }
            }
        }

        public byte[] GetOutput()
        {
            int numCoordinates = 4;
            byte[] output = new byte[numCoordinates];

            if (0 < NumberOfPeopleInShot())
            {
                if (!PersonInPicture(trackedPerson))
                {
                    ChoosePerson();
                }
                output[0] = (byte)trackedPerson.ID;
                output[1] = (byte)trackedPerson.x;
                output[2] = (byte)trackedPerson.y;
                output[3] = (byte)trackedPerson.z;
            }
            else
            {
                output[0] = (byte)0;
                output[1] = (byte)0;
                output[2] = (byte)0;
                output[3] = (byte)0;
            }
            return output;
        }

        private bool PersonChosen(Person p)
        {
            if (p.ID != 0 && p.ID == trackedPerson.ID)
            {
                return true;
            }
            return false;
        }

        private bool PersonInPicture(Person p)
        {
            foreach (Person person in people)
            {
                if (p.ID != 0 && p.ID == person.ID)
                {
                    return true;
                }
            }
            return false;
        }

        private void ChoosePerson()
        {
            ChooseRandomPerson();
        }

        private void ChooseRandomPerson()
        {
            int n = 0;

            if ((n = NumberOfPeopleInShot()) != 0)
            {
                do
                {
                    trackedPerson = people[rand.Next(0, people.Length)];
                    trackedPersonID = trackedPerson.ID;
                } 
                while (trackedPerson.ID == 0);
                
            }
        }

        private void ChooseNearestPerson()
        {
            int n = 0;
            Person nearestPerson = new Person(0, 999, 999, 999);

            if ((n = NumberOfPeopleInShot()) != 0)
            {
                for (int i = 0; i < people.Length; i++)
                {
                    if (people[i].ID != 0 && people[i].z < nearestPerson.z)
                    {
                        nearestPerson = people[i];
                    }
                }
            }
            if (nearestPerson.ID != 0)
            {
                trackedPerson = nearestPerson;
                trackedPersonID = trackedPerson.ID;
            }
        }

        private int NumberOfPeopleInShot()
        {

            int n = 0;

            for (int i = 0; i < 6; i++)
            {
                if (people[i].ID != 0)
                {
                    n++;
                }
            }

            return n;

        }

        static void Main(string[] args)
        {
            DecisionMaker decisionMaker = new DecisionMaker();
        }

    }
}
