﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Media.Animation;

namespace DecisionMaker
{
    class FrameReceiver : IPositionInfoReleased, IConnectionFailed
    {
        //eventhandler tracking data
        event EventHandler newPositonInfoEvent;
        object newPositonInfoEventLock = new Object();

        event EventHandler IPositionInfoReleased.OnNewPositionInfoArrived
        {
            add
            {
                lock (newPositonInfoEventLock)
                {
                    newPositonInfoEvent += value;
                }
            }
            remove
            {
                lock (newPositonInfoEventLock)
                {
                    newPositonInfoEvent -= value;
                }
            }
        }

        //eventhandler connection failed
        event EventHandler connectionFailedEvent;
        object connectionFailedEventLock = new Object();

        event EventHandler IConnectionFailed.OnConnectionFailed
        {
            add
            {
                lock (connectionFailedEventLock)
                {
                    connectionFailedEvent += value;
                }
            }
            remove
            {
                lock (connectionFailedEventLock)
                {
                    connectionFailedEvent -= value;
                }
            }
        }

        enum FrameType
        {
            body = 0,
            color = 1,
            depth = 2,
            infrared = 3
        }

        //network
        private TcpClient tcpClient;
        private IPEndPoint serverEndPoint = new IPEndPoint(IPAddress.Parse("10.0.0.6"), 8001);

        //thread
        private Thread frameReceiver;

        private bool setup;

        private byte[] framemessage = { Convert.ToByte('b'), Convert.ToByte('o'), Convert.ToByte('d'), Convert.ToByte('y') };

        public FrameReceiver()
        {
            
            tcpClient = new TcpClient();

            Console.WriteLine("Connecting to TCP server at IP: " + serverEndPoint.Address.ToString());
            if (AttemptConnectToServer() == -1)
            {
                setup = false;

                EventHandler connectionEventHandler = connectionFailedEvent;
                if (connectionEventHandler != null)
                {
                    byte[] m = { Convert.ToByte('f'), Convert.ToByte('r'), Convert.ToByte('a'), Convert.ToByte('m'), Convert.ToByte('e') };
                    connectionEventHandler(this, new ConnectionFailedEventArgs(m));
                }
                return;
            }
            Console.WriteLine("Connect to TCP server succeeded");

            Console.WriteLine("Setting up the body frame receiver");
            frameReceiver = new Thread(new ThreadStart(ReceiveBodyFrame));
            frameReceiver.Start();

            Console.WriteLine("Setup of body frame receiver succeeded\n");

            setup = true;
        }

        public bool CheckSetup()
        {
            return setup;
        }
        
        private int AttemptConnectToServer()
        {
            TimeSpan starttime = (TimeSpan)DateTime.Now.TimeOfDay;
            TimeSpan interval = new TimeSpan(0, 0, 6);

            bool connected = false;
            bool timedout = false;

            while (connected == false && timedout == false)
            {
                if ((TimeSpan)DateTime.Now.TimeOfDay < (starttime + interval))
                {
                    connected = ConnectToServer();
                }
                else
                {
                    timedout = true;
                }
            }

            if (connected == false)
            {
                return -1;
            }
            else
            {
                return 0;
            }
        }

        private bool ConnectToServer()
        {
            try
            {
                if (tcpClient.Connected)
                {
                    tcpClient.Close();
                }

                tcpClient = null;
                tcpClient = new TcpClient();

                tcpClient.Connect(serverEndPoint);

                NetworkStream stream = tcpClient.GetStream();

                stream.Write(framemessage, 0, framemessage.Length);
                stream.Flush();
            }
            catch
            {
                return false;
            }
            return true;
        }

        private void ReceiveBodyFrame()
        {

            int numberOfPeople = 6;
            int slotsPerPerson = 4;
       
            NetworkStream stream = tcpClient.GetStream();

            int messagelength = numberOfPeople * slotsPerPerson;
            byte[] bytemessage = new byte[messagelength + 1];
            int bytesRead = 0;

            while (true)
            {

                Console.WriteLine("Receiving new body frame");

                try
                {
                    stream = tcpClient.GetStream();
                    bytesRead = stream.Read(bytemessage, 0, messagelength + 1);
                }
                catch
                {
                    Console.WriteLine("Could not receive message. Assuming server has been disconnected\n");

                    if (AttemptConnectToServer() == -1)
                    {
                        EventHandler connectionEventHandler = connectionFailedEvent;
                        if (connectionEventHandler != null)
                        {
                            byte[] m = { Convert.ToByte('f'), Convert.ToByte('r'), Convert.ToByte('a'), Convert.ToByte('m'), Convert.ToByte('e') };
                            connectionEventHandler(this, new ConnectionFailedEventArgs(m));
                        }
                    }
                }

                if (bytemessage[0] == (byte)FrameType.body)
                {

                    byte[] message = new byte[messagelength];

                    for (int i = 0; i < messagelength; i++)
                    {
                        message[i] = bytemessage[i + 1];
                    }

                    Console.WriteLine("New body frame received");

                    PrintMessage(message);
                    Console.Write("\n");

                    EventHandler newPositionInfoEventHandler = newPositonInfoEvent;
                    if (newPositionInfoEventHandler != null)
                    {
                        newPositionInfoEventHandler(this, new PositionInfoEventArgs(message));
                    }
                }
            }
        }

        void PrintMessage(byte[] message)
        {
            for (int i = 0; i < message.Length; i++)
            {
                Console.Write(message[i]);
            }

            Console.Write("\n");

        }
    }
}
