﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace FrameSupplier
{

    enum FrameType
    {
        body = 0,
        color = 1,
        depth = 2,
        infrared = 3
    }

    public struct ClientStruct
    {
        public TcpClient client;
        public string frameType;

        public ClientStruct(TcpClient c, string ft)
        {
            client = c;
            frameType = ft;
        }
    }

    class KinectFrameServer
    {

        private const int ColorFrameWidth = 1920;
        private const int ColorFrameHeight = 1080;

        private const int DepthFrameWidth = 512;
        private const int DepthFrameHeight = 424;

        private const int InfraredFrameWidth = 512;
        private const int InfraredFrameHeight = 424;

        private TcpListener tcpListener;
        private Thread clientListener;
        private List<ClientStruct> clients;

        private FrameSupplier frameSupplier;

        public KinectFrameServer()
        {
            Console.WriteLine("--------------------------------------------------" +
                              "\n    Setup of module FRAME SUPPLIER initiated" + 
                              "\n--------------------------------------------------\n");

            frameSupplier = new FrameSupplier();
            if (frameSupplier.SensorOn() == false)
            {
                Console.WriteLine("Sensor not connected");
                return;
            }

            Console.WriteLine("Starting TCP listener");
            tcpListener = new TcpListener(IPAddress.Any, 8001);
            tcpListener.Start();

            Console.WriteLine("Start listening for clients");
            clientListener = new Thread(new ThreadStart(ListenForClients));
            clientListener.Start();
            clients = new List<ClientStruct>();

            Console.WriteLine("\n--------------------------------------" +
                              "\n         TCP setup completed" +
                              "\n--------------------------------------\n");

            Console.WriteLine("\n--------------------------------------------------" +
                              "\n    Setup of module FRAME SUPPLIER completed" +
                              "\n--------------------------------------------------\n");
        }

        private void Terminate()
        {
            clientListener.Abort();
            clientListener.Join();
            tcpListener.Stop();
            return;
        }

        private void AddClient(TcpClient c, string ft)
        {
            Console.WriteLine("Adding client " + ((IPEndPoint)(c).Client.RemoteEndPoint).Address.ToString());

            clients.Add(new ClientStruct(c, ft));

            Console.WriteLine("Number of connected clients: " + clients.Count + "\n");
        }

        private void RemoveClient(TcpClient c)
        {
            Console.WriteLine("Removing client " + ((IPEndPoint)(c).Client.RemoteEndPoint).Address.ToString());

            for (int i = 0; i < clients.Count; i++)
            {
                if (clients[i].client == c)
                {
                    clients.RemoveAt(i);
                }
            }

            Console.WriteLine("Number of connected clients: " + clients.Count + "\n");
        }

        private void ListenForClients()
        {

            TcpClient new_tcpclient;
            NetworkStream stream;
            byte[] bytemessage;
            int bytesRead;

            while (true)
            {
                Console.WriteLine("Waiting for clients to connect");
                new_tcpclient = tcpListener.AcceptTcpClient();

                Console.WriteLine("New client attempting to connect: " + ((IPEndPoint)(new_tcpclient).Client.RemoteEndPoint).Address.ToString());

                stream = new_tcpclient.GetStream();

                bytemessage = new byte[8];
                bytesRead = 0;

                Console.WriteLine("Receiving message from client ");

                try
                {
                    bytesRead = stream.Read(bytemessage, 0, bytemessage.Length);
                }
                catch
                {
                    Console.WriteLine("Could not receive message");
                }

                if (bytesRead == 0)
                {
                    Console.WriteLine("No message to receive");
                }

                string message = System.Text.Encoding.UTF8.GetString(bytemessage);
                message = StripMessage(message);                    
                Console.WriteLine("Received message: " + message);

                Thread new_clientThread;

                if (message == "body" || message == "color" || message == "depth" || message == "infrared")
                {
                    Console.WriteLine("Message approved. Accepting clients request to connect\n");
                    AddClient(new_tcpclient, message);
                    ClientStruct new_clientstruct = new ClientStruct(new_tcpclient, message);
                    new_clientThread = new Thread(new ParameterizedThreadStart(SendFrameInfo));
                    new_clientThread.Start(new_clientstruct);
                }
                else
                {
                    Console.WriteLine("Message not accepted. Rejecting client");
                }

            }
            
        }

        private void SendFrameInfo(object o)
        {
            ClientStruct tcpclientstruct = (ClientStruct)o;
            int messageSize = 0;
            int frametype = 0;

            switch (tcpclientstruct.frameType)
            {
                case "body":
                    messageSize = 6 * 4;
                    frametype = 0;
                    break;
                case "color":
                    messageSize = ColorFrameWidth * ColorFrameHeight * ((PixelFormats.Bgr32.BitsPerPixel + 7) / 8);
                    frametype = 1;
                    break;
                case "depth":
                    messageSize = DepthFrameWidth * DepthFrameHeight * ((PixelFormats.Bgr32.BitsPerPixel + 7) / 8);
                    frametype = 2;
                    break;
                case "infrared":
                    messageSize = InfraredFrameWidth * InfraredFrameHeight * ((PixelFormats.Bgr32.BitsPerPixel + 7) / 8);
                    frametype = 3;
                    break;
            }

            byte[] frame = new byte[messageSize];
            byte[] message = new byte[messageSize + 1];

            TcpClient tcpClient = tcpclientstruct.client;
            NetworkStream stream = tcpClient.GetStream();

            while (true)
            {
                message = new byte[messageSize + 1];
                frame = new byte[messageSize];
                frame = frameSupplier.GetByteArray(tcpclientstruct.frameType);
                message[0] = (byte)frametype;
                for (int i = 0; i < messageSize; i++)
                {
                    message[i + 1] = frame[i];
                }

                    try
                    {
                        Console.WriteLine("Sending " + tcpclientstruct.frameType.ToString() + "frame to client " +
                                          ((IPEndPoint)(tcpclientstruct.client).Client.RemoteEndPoint).Address.ToString());

                        stream.Write(message, 0, message.Length);
                        stream.Flush();
                    }
                    catch
                    {
                        Console.WriteLine("Message could not be sent. \n" +
                                          "Client " + ((IPEndPoint)(tcpclientstruct.client).Client.RemoteEndPoint).Address.ToString()
                                          + " assumed to be disconnected\n");
                        RemoveClient(tcpclientstruct.client);
                        return;
                    }

                Console.WriteLine("Message successfully sent\n");

                message = null;

                Thread.Sleep(1000);
            }
        }

        static void Main()
        {
           KinectFrameServer frameserver = new KinectFrameServer();
        }

        public string StripMessage(string m)
        {
            string message = "";
            for (int i = 0; i < m.Length; i++)
            {
                if ('a' <= m[i] && m[i] <= 'z')
                {
                    message += m[i];
                }
            }
            return message;
        }

    }
}
